local util = include( "modules/util" )

-- init will be called once
local function init( modApi )

	local dataPath = modApi:getDataPath()
	local scriptPath = modApi:getScriptPath()
	
	include( scriptPath .. "/lib_sequentialModLoader" )( modApi )
	modApi.requirements = { "Contingency Plan" }
	KLEIResourceMgr.MountPackage( dataPath .. "/gui.kwad", "data" )
	
	local agent_list = include( scriptPath .. "/agent_list" );
	-- Add generation options (on-file)
	for _, k in ipairs( agent_list.AGENT_LIST ) do
		local name = STRINGS.RAA.NAMES_UPPER[string.upper(k)];
		modApi:addGenerationOption( k .. "_o",  name .. STRINGS.RAA.OPTIONS.ENABLE_AGENT_P_O, STRINGS.RAA.OPTIONS.ENABLE_AGENT_TIP_O )
	end
	-- Add generation options (archive)
	for _, k in ipairs( agent_list.AGENT_LIST ) do
		local name = STRINGS.RAA.NAMES_UPPER[string.upper(k)];
		modApi:addGenerationOption( k .. "_a",  name .. STRINGS.RAA.OPTIONS.ENABLE_AGENT_P_A, STRINGS.RAA.OPTIONS.ENABLE_AGENT_TIP_A )
	end
	-- Add generation options (dlc)
	for _, k in ipairs( agent_list.AGENT_LIST_DLC ) do
		local name = STRINGS.RAA.NAMES_UPPER[string.upper(k)];
		modApi:addGenerationOption( k .. "_dlc",  name .. STRINGS.RAA.OPTIONS.ENABLE_AGENT_P_D, STRINGS.RAA.OPTIONS.ENABLE_AGENT_TIP_D )
	end
end

local function removeAgent( agency, agentDef )
	-- Remove all potentials with the same ID (or template)
	for i = #agency.unitDefsPotential, 1, -1 do
		local def2 = agency.unitDefsPotential[ i ]
		if (def2.id == agentDef.id) or (def2.template == agentDef.template) then
			table.remove( agency.unitDefsPotential, i )
		end
	end
end

-- load may be called multiple times with different options enabled
local function load( modApi, options )
	local scriptPath = modApi:getScriptPath() 
	local serverdefs = include ( scriptPath .. "/serverdefs" )
	
	-- DRM'ed items
	local itemdefs = include( scriptPath .. "/itemdefs" )
	for name, itemDef in pairs(itemdefs) do
		modApi:addItemDef( name, itemDef )
	end
	
	local agent_list = include( scriptPath .. "/agent_list" );
	
	for _, k in ipairs( agent_list.AGENT_LIST ) do
		local is_o = options[k .. '_o'].enabled
		local is_a = options[k .. '_a'].enabled
		-- agents cannot co-exist in both states, so if player picked both - choose one random
		if is_o and is_a then
			local rnd = math.random()
			log:write('DICE for %s = %f', k, rnd);
			if rnd < 0.5 then
				is_o = false
			else
				is_a = false
			end
		end
		-- remove on-file agent
		if not is_o then 
			log:write( 'RAA: Remove agent %s', k );
			removeAgent( serverdefs.ORIGINAL_AGENCY, serverdefs.TEMPLATE_AGENCY.unitDefsPotential[k] )
		end
		-- add archive agent
		if is_a then 
			log:write( 'RAA: Add archive agent %s', k );
			modApi:addRescueAgent( serverdefs.TEMPLATE_AGENCY.unitDefsPotential[k] )
		end
	end
	-- Remove DLC agents (their prototypes may not exist, so we remove them by name. Also, they have no archive version)
	-- Luckily, their names are normal, not 'stealth_1' etc.
	for _, k in ipairs( agent_list.AGENT_LIST_DLC ) do
		if not options[k .. '_dlc'].enabled then 
			log:write( 'RAA: Remove DLC agent %s', k );
			removeAgent( serverdefs.ORIGINAL_AGENCY, {id=-1, template=k} )
		end
	end
	util.tlog(serverdefs.TEMPLATE_AGENCY.unitDefsPotential, 'L_TEMPLATE_AGENCY:   ')
	util.tlog(serverdefs.ORIGINAL_AGENCY.unitDefsPotential, 'G_TEMPLATE_AGENCY:   ')
end

local function initStrings( modApi )
	local dataPath = modApi:getDataPath()
	local scriptPath = modApi:getScriptPath()
	local DLC_STRINGS = include( scriptPath .. "/strings" )
	modApi:addStrings( dataPath, "RAA", DLC_STRINGS )  
end
 
return {
    init = init,
    load = load,
    initStrings = initStrings,
}
